@@ -5,9 +5,9 @@
 
 #define WIDTH 100
 #define HEIGHT 100
-const char *text = "Hamburgerfonts\nวิวิวิวิวิวิ\nبهداد";
+const char *text = "The quick brown fox jumped over the lazy dog!";
 
-int num_iters = 50;
+int num_iters = 100;
 
 GMutex mutex;
 
@@ -35,7 +35,7 @@ draw (cairo_t *cr, PangoLayout *layout, unsigned int i)
   cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 
   cairo_identity_matrix (cr);
-  cairo_scale (cr, (100 + i) / 100.,  (100 + i) / 100.);
+  cairo_scale (cr, (100 + i) / 10.,  (100 + i) / 10.);
   pango_cairo_update_layout (cr, layout);
 
   pango_cairo_show_layout (cr, layout);
@@ -68,7 +68,7 @@ thread_func (gpointer data)
 int
 main (int argc, char **argv)
 {
-  int num_threads = 5;
+  int num_threads = 100;
   int i;
   GPtrArray *threads = g_ptr_array_new ();
   GPtrArray *surfaces = g_ptr_array_new ();
@@ -89,7 +89,7 @@ main (int argc, char **argv)
       char buf[10];
       cairo_surface_t *surface = create_surface ();
       g_ptr_array_add (surfaces, surface);
-      g_snprintf (buf, sizeof (buf), "%d", i);
+      snprintf (buf, sizeof (buf), "%d", i);
       g_ptr_array_add (threads,
 		       g_thread_new (buf,
 				     thread_func,
